const express = require("express");
const router = express.Router();
const multer = require("multer");
const db_con = require("../connection");

const storage = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, "uploads/");
  },
  filename: function (req, file, cb) {
    cb(null, Date.now() + file.originalname);
  },
});

let upload = multer({
  storage: storage,
});

router.get("/", (req, res, next) => {
  res.send("Hellow world");
  console.log("Tour Added");
});

router.post("/addNewVendor", (req, res) => {
  const userBody = req.body;
  let query =
    "INSERT INTO all_vendors (`user_name`,`user_email`,`user_mobile`,`user_display_name`,`user_display_Id`,`user_password`,`user_color`,`user_type`) VALUES (?,?,?,?,?,?,?,?)";

  db_con.query(
    query,
    [
      userBody.user_name,
      userBody.user_email,
      userBody.user_mobile,
      userBody.user_display_name,
      userBody.user_display_Id,
      userBody.user_password,
      userBody.user_color,
      userBody.user_type,
    ],
    (error, results, fields) => {
      if (error) {
        console.log(error);
        res.send({
          code: 400,
          failed: "error request",
        });
      } else {
        console.log("Vendor Added Successfully");
        res.send({
          code: 200,
          message: "Vendor Added Successfully",
        });
      }
    }
  );
});

router.post("/vendorlogin", (req, res) => {
  const body = req.body;

  query = "SELECT * FROM all_vendors WHERE user_email=? AND user_password=?";
  db_con.query(
    query,
    [body.email, body.password],
    function (error, results, fields) {
      if (error) {
        console.log(error);
        res.send({
          code: 402,
          failed: "error request",
        });
      } else {
        if (results.length != 0) {
          res.send({
            code: 200,
            message: "Login Successful",
            Payload: results,
          });
        } else {
          res.send({
            code: 402,
            message: "No Data Found",
          });
        }
      }
    }
  );
});

const id = (length) => {
  let mask = "";
  mask += "0123456789";
  let result = "";
  for (var i = length; i > 0; --i)
    result += mask[Math.round(Math.random() * (mask.length - 1))];
  return result;
};

// get api
router.get("/getAllVendor", (req, res) => {
  query = "SELECT * FROM all_vendors";
  db_con.query(query, function (error, results, fields) {
    if (error) {
      res.send({
        code: 400,
        failed: "error request",
      });
    } else {
      res.send({
        code: 200,
        message: "Success",
        Payload: results,
      });
    }
  });
});

router.post("/deleteCategory", (req, res) => {
  const body = req.body;
  // let query = `DELETE FROM all_tour_category WHERE category_id = ?`;
  category_id_container = [];
  console.log(req.body);
  req.body.forEach((element) => {
    category_id_container.push(element.category_id);
  });
  let allids = category_id_container.join(",");
  let query =
    `DELETE FROM all_tour_category WHERE category_id IN (` + allids + `);`;
  db_con.query(query, (error, results, fields) => {
    if (error) {
      console.log("error ocurred", error);
      res.send({
        code: 400,
        failed: "error request",
      });
    } else {
      console.log("Resultats: ", results);
      res.send({
        code: 200,
        message: "Deleted Successfully",
      });
    }
  });
});

// vandor payment controller

router.post("/addNewVendorPayout", (req, res) => {
  const userBody = req.body;
  let query =
    "INSERT INTO vendor_payout (`agent_id`,`tour_agent_name`,`booking_id`,`booking_total_price`,`commission_percentage`,`vendor_total_payout`,`vendor_payout_status`,`vendor_payout_date`,`user_payout_date`) VALUES (?,?,?,?,?,?,?,?,?)";

  db_con.query(
    query,
    [
      userBody.agent_id,
      userBody.tour_agent_name,
      userBody.booking_id,
      userBody.booking_total_price,
      userBody.commission_percentage,
      userBody.vendor_total_payout,
      userBody.vendor_payout_status,
      userBody.vendor_payout_date,
      userBody.user_payout_date,
    ],
    (error, results, fields) => {
      if (error) {
        console.log(error);
        res.send({
          code: 400,
          failed: "error request",
        });
      } else {
        console.log("Vendor Payout Added Successfully");
        res.send({
          code: 200,
          message: "Vendor Payout Added Successfully",
        });
      }
    }
  );
});

// // get api
// router.get('/getAllVendorPayout', (req, res) => {
//     query = "SELECT * FROM vendor_payout"
//     db_con.query(query, function (error, results, fields) {
//         if (error) {
//             res.send({
//                 "code": 400,
//                 "failed": "error request"
//             })
//         } else {
//             res.send({
//                 "code": 200,
//                 "message": "Success",
//                 "Payload": results
//             });
//         }
//     });
// })

// // get api

router.get("/getAllVendorPayout", (req, res) => {
  const vendortype = req.query.vendortype;
  const vendorId = req.query.vendorId;
  console.log(vendortype);
  if (vendortype == "Admin") {
    query = "SELECT * FROM vendor_payout";
    db_con.query(query, function (error, results, fields) {
      if (error) {
        res.send({
          code: 400,
          failed: "error request",
        });
      } else {
        res.send({
          code: 200,
          message: "Success",
          Payload: results,
        });
      }
    });
  } else {
    query = "SELECT * FROM vendor_payout WHERE agent_id=?";
    db_con.query(query, [vendorId], function (error, results, fields) {
      if (error) {
        console.log(error);
        res.send({
          code: 402,
          failed: "error request",
        });
      } else {
        if (results.length != 0) {
          res.send({
            code: 200,
            message: "Login Successful",
            Payload: results,
          });
        } else {
          res.send({
            code: 402,
            message: "No Data Found",
          });
        }
      }
    });
  }
});

// update api

router.post("/updateVendorPayout", (req, res) => {
  let query = `UPDATE vendor_payout SET vendor_payout_status = ?, vendor_payout_date = ? WHERE booking_id = ?; `;
  db_con.query(
    query,
    [
      req.body.vendor_payout_status,
      req.body.vendor_payout_date,
      req.body.booking_id,
    ],
    (error, results, fields) => {
      if (error) {
        console.log("error ocurred", error);
        res.send({
          code: 400,
          failed: "error request",
        });
      } else {
        res.send({
          code: 200,
          message: "Updated Successfully",
        });
      }
    }
  );
});

module.exports = router;
